﻿using System;
using System.Data.Entity;
using System.Threading.Tasks;
using ImageProcessing.DistributedAlgorithms.MarkerGenerating;
using YSoft.Rqa.VisualProwess.AutoCalibration.Db.Entities.Base;

namespace YSoft.Rqa.VisualProwess.AutoCalibration.Db.Entities
{
    public class Device : IdBase
    {
        public int DeviceModelId { get; set; }
        public double Recessing { get; set; }
        public int MinDistanceAroundScreen { get; set; }
        public MarkerTemplate GetMarkerTemplate => new MarkerTemplate(MinDistanceAroundScreen);

        public static Device Create(int deviceModelId, int minDistanceAroundScreen)
        {
            if (minDistanceAroundScreen < 2)
                throw new ArgumentOutOfRangeException("Cannot create new Device. Minimal distance around screen must be at least 2 milimeters.");
              
            var device = new Device
            {
                DeviceModelId = deviceModelId,
                MinDistanceAroundScreen = minDistanceAroundScreen,
                Recessing = 0
            };
            return device;
        }

        public static async Task<Device> GetById(int id, CalibrationContext context)
        {
            var device = await context.Devices.FirstOrDefaultAsync(dev => dev.DeviceModelId == id);
            if (device == null)
                throw new ArgumentOutOfRangeException($"There is no device with DeviceModelId = {id}.");
            return device;
        }
    }
}