﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Threading.Tasks;
using ImageProcessing.DistributedAlgorithms.MarkerGenerating;
using OpenCvSharp;
using YSoft.Rqa.VisualProwess.AutoCalibration.Db.Entities.Base;
using Size = YSoft.Rqa.VisualProwess.AutoCalibration.Db.Entities.Base.Size;

namespace YSoft.Rqa.VisualProwess.AutoCalibration.Db.Entities
{
    public class Robot :IdBase
    {
        public int ChessboardModelId { get; set; }
        public virtual ChessboardModel ChessboardModel { get; set; }
        public Vector3d TranslationChessboardToArm { get; set; }

        public static async Task<Robot> GetById(int id, CalibrationContext context)
        {
            var robot = await context.Robots.SingleAsync(r => r.Id == id);
            if (robot == null)
                throw new ArgumentOutOfRangeException($"There is no robot with id [{id}].");
            return robot;
        }

        public double[,] GetTransformMatrixFromChessboardToArm()
        {
            var tMat = new double[4, 4];
            for (int i = 0; i < tMat.GetLength(0); i++)
            {
                tMat[i, i] = 1;
            }
            tMat[0, 3] = TranslationChessboardToArm.X;
            tMat[1, 3] = TranslationChessboardToArm.Y;
            tMat[2, 3] = TranslationChessboardToArm.Z;

            return tMat;
        }
        
        public List<Scalar> GetChessboardPoints()
        {
            var chessboard = new List<Point2d>();
            var squareSize = ChessboardModel.SquareSize;
            for (var i = ChessboardModel.NumberOfInnerPoints.Height - 1; i >= 0; i--)
            //for (int i = 0; i < ChessboardModel.NumberOfInnerPoints.Height; i++)
            {
                for (int j = 0; j < ChessboardModel.NumberOfInnerPoints.Width; j++)
                {
                    chessboard.Add(new Point2d(j * squareSize, i * squareSize));
                }
            }
            return chessboard.Select(p => new Scalar(p.X, p.Y, 0, 1)).ToList();
        }

        public static void AddToDb(double sizeX, double sizeY, double squareSize, double transX, double transY,
            double transZ, CalibrationContext context)
        {
            context.Robots.Add(new Robot()
                {
                    ChessboardModel = new Db.Entities.ChessboardModel()
                    {
                        SquareSize = squareSize,
                        NumberOfInnerPoints = new Size(sizeX, sizeY)
                    },
                    TranslationChessboardToArm = new Vector3d(transX, transY, transZ)
                }
                );
        }
    }
}