﻿using System;
using System.Collections.Generic;
using System.Linq;
using OpenCvSharp;

namespace YSoft.Rqa.VisualProwess.AutoCalibration.DetectionRefine
{
    internal class RefinedEdge
    {
        private List<Sigmoid> Sigmoids { get; set; }

        internal RefinedEdge(List<Sigmoid> sigmoids)
        {
            Sigmoids = sigmoids;
        }

        internal static Line2D Interpolate(List<Sigmoid> sigmoids, double percent)
        {
            return Cv2.FitLine(
                GetFirstInflexPointsByRmse(sigmoids, percent),
                DistanceTypes.L2, 0, .01, .01);
        }

        private static List<Point2f> GetFirstInflexPointsByRmse(List<Sigmoid> sigmoids, double percent)
        {
            return sigmoids
                .Where(s => !double.IsNaN(s.GetRmse()) && !double.IsNaN(s.InflexPoint.X) && !double.IsNaN(s.InflexPoint.Y))
                .OrderBy(x => x.GetRmse())
                .Take((int)(sigmoids.Count*percent/100))
                .Select(x=>x.InflexPoint)
                .ToList();
        }


    }
}