﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using OpenCvSharp;
using YSoft.Rqa.VisualProwess.AutoCalibration.CameraCalibration;
using YSoft.Rqa.VisualProwess.AutoCalibration.Db;
using YSoft.Rqa.VisualProwess.AutoCalibration.Exceptions;

namespace YSoft.Rqa.VisualProwess.AutoCalibration.DiplomkaTests
{
    public class CameraCalibTester
    {
        public static async Task TestForOneDevice(List<Mat> images, Size size, int devModelId, CalibrationContext context)
        {
            var iter = 0;
            var calibrated = 0;
            var calibError = 0;
            var markerError = 0;
            var otherErr = 0;
            foreach (var img in images)
            {
                Console.Write(iter++);
                try
                {
                    var calibrator = new CameraCalibrator(context);
                    var res = await calibrator.Calibrate(devModelId, size, img);
                    calibrated++;
                    Console.WriteLine(" calibrated");
                }
                catch (ValidationCalibrationException)
                {
                    calibError++;
                    Console.WriteLine(" calib error");
                }
                catch (MarkerCannotBeDetectedException)
                {
                    markerError++;
                    Console.WriteLine(" marker error");
                }
                catch
                {
                    otherErr++;
                    Console.WriteLine(" other error");
                }

                Cv2.DestroyAllWindows();
            }
        }

        public static void GetData(out List<List<Mat>> images, out List<Size> sizes, out List<int> devIds)
        {
            var fld =
                @"C:\Users\adamek\Code\ysoft\improc\src\AutoCalibration\AutoCalibration\shots\diplomkaTest\danovo\";
            var filesBrother = Directory.GetFiles(fld + "Brother", "*.jpg", SearchOption.AllDirectories);
            var filesKm = Directory.GetFiles(fld + "Km", "*.jpg", SearchOption.AllDirectories);
            var filesOki = Directory.GetFiles(fld + "Oki", "*.jpg", SearchOption.AllDirectories);
            var filesSharp = Directory.GetFiles(fld + "Sharp", "*.jpg", SearchOption.AllDirectories);

            var imagesBrother = filesBrother.Select(file => Cv2.ImRead(file)).ToList();
            var imagesKm = filesKm.Select(file => Cv2.ImRead(file)).ToList();
            var imagesOki = filesOki.Select(file => Cv2.ImRead(file)).ToList();
            var imagesSharp = filesSharp.Select(file => Cv2.ImRead(file)).ToList();
            images = new List<List<Mat>> {imagesBrother, imagesKm, imagesOki, imagesSharp};

            var szBrother = new Size(109,61);
            var szKm = new Size(198,112);
            var szOki = new Size(154,86);
            var szSharp = new Size(224,125);
            sizes = new List<Size> {szBrother, szKm, szOki, szSharp};

            devIds = new List<int> {48, 4, 33, 24};
        }
    }
}