﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using FSLibrary;
using OpenCvSharp;
using YSoft.Rqa.VisualProwess.AutoCalibration.Db;
using YSoft.Rqa.VisualProwess.AutoCalibration.MarkerDetecting;
using YSoft.Rqa.VisualProwess.AutoCalibration.RobotCalibration;

namespace YSoft.Rqa.VisualProwess.AutoCalibration.DiplomkaTests
{
    public class RefineTester
    {
        public static async Task<List<double>> Test(List<Mat> imgs, Size screenSize, int devId, int robId, int camId,
            CalibrationContext context, bool refine)
        {
            var averages = new List<double>();
            var rbtCalib = new RobotCalibrator(context);
            foreach (var img in imgs)
            {
                await rbtCalib.Init(devId,screenSize,robId,camId,img);
                var markerTemplate = rbtCalib.Device.GetMarkerTemplate;
                try
                {
                    var marker = new MarkerDetector(markerTemplate, img).Detect(refine);
                    var prms = rbtCalib.ScreenToCamera(refine);
                    var error = matrixTransform.backproject_error(prms, rbtCalib.Camera.GetIntrinsicMatrix(),
                        rbtCalib.Device.GetMarkerTemplate.GetMarkerModel(screenSize), rbtCalib.ExtractAllPlanePoints(marker));

                    double average = 0;
                    for (int i = 0; i < 20; i++)
                    {
                        average += Math.Sqrt(Math.Pow(error[0, i], 2) + Math.Pow(error[1, i], 2));
                    }
                    averages.Add(average / 20);
                }
                catch{}
            }
            return averages;
        }
    }
}